@@ -68,7 +68,7 @@ main(argc, argv)
 	char	*configfile, *pooption, *path, *mailpriority;
 	FILE	*vfp;
 	struct passwd *pwd;
-	char	*LC_CTYPE;
+	char	*LC_ctype;
 	char	buf[8192];
 	char   *buf2 = NULL;
 
@@ -82,7 +82,7 @@ main(argc, argv)
 	if (mailpriority) mail_priority = atoi(mailpriority);
 
 	/* Pick up the sender's idea about C-chartype.. */
-	LC_CTYPE = getenv("LC_CTYPE");
+	LC_ctype = getenv("LC_CTYPE");
 
 	printq = strcmp(progname, MAILQ) == 0;
 	aliases = strcmp(progname, NEWALIASES) == 0;
@@ -96,7 +96,7 @@ main(argc, argv)
 	configfile = pooption = NULL;
 	if (printq || aliases)
 		goto otherprog;
-	while ((c = getopt(argc, argv,"EJb:e:r:f:h:F:C:d:Oo:P:imnqstvV")) != EOF) {
+	while ((c = getopt(argc, argv,"EJb:e:r:f:h:F:C:d:Oo:P:imnqstvVx")) != EOF) {
 		switch (c) {
 		case 'J': break; /* Sony NEWS OS  JIS-conversion option,
 				    ignore */
@@ -203,6 +203,8 @@ main(argc, argv)
 			prversion("sendmail");
 			exit(0);
 			break;
+		case 'x':	/* Ignore this AIXism */
+			break;
 		default:
 			++errflg;
 			break;
@@ -483,6 +485,8 @@ otherprog:
 		if (vfd >= 0) {
 			if (fork() > 0) exit(EX_OK); /* Let the child to do
 							the trace printout */
+			if (vfd != 0) close(0);
+			if (vfd != 1) close(1);
 			vfp = fdopen(vfd, "r");
 			while (vfp != NULL) {
 				while (fgets(buf, sizeof buf, vfp) != NULL)
