@@ -149,10 +149,11 @@ bgsetup(c)
  */
 
 int
-execute(c, caller, oretcode)
+execute(c, caller, oretcode, name)
 	struct osCmd	*c,		/* function we are about to execute */
 			*caller;	/* the function we are currently in */
 	int		oretcode;	/* default value for return code */
+	char		*name;		/* Redundant function name info */
 {
 	int ac = 0;			/* argument count in av[] */
 	char **av = NULL;		/* argument vector, with ac entries */
@@ -317,7 +318,7 @@ execute(c, caller, oretcode)
 				BEGINGRANDCHILD(pid)
 				while (c->rval != NULL) {
 					s_grind(c->rval, stdout);
-					if (c->rval = cdr(c->rval))
+					if ((c->rval = cdr(c->rval)))
 						putchar(' ');
 					else
 						break;
@@ -344,7 +345,7 @@ execute(c, caller, oretcode)
 				tmp = c->rval;
 				while (c->rval != NULL) {
 					s_grind(c->rval, stdout);
-					if (c->rval = cdr(c->rval))
+					if ((c->rval = cdr(c->rval)))
 						putchar(' ');
 					else
 						break;
@@ -672,7 +673,7 @@ sb_push(siopp, sioptop)
 	siop->sb_base = NULL;
 	siop->sb_bufsiz = -1;
 	siop->sb_refcnt = 0;
-	siop->sb_flag = -1;	/* magic value */
+	siop->sb_flag = (short)-1;	/* magic value */
 	if (sioptop != NULL) {
 		while (sioptop != NULL && sioptop != sioptop->_sb_data)
 			sioptop = sioptop->_sb_data;
@@ -749,14 +750,14 @@ sb_free(n)
 /*std_printf("FREE(%d) = %x\n", n, siofds[n]);*/
 	siop = siofds[n];
 	if (siop == NULL) {
-		fprintf(stderr, "%s: no siofds to free\n",
-				progname);
+		fprintf(stderr, "%s: no siofds to free (fd=%d)\n",
+				progname, n);
 		return;
 	}
 	siofds[n] = siop->sb_next;
 	flag = siop->_sb_data != siop;
 
-	siop->sb_refcnt--;
+	siop->sb_refcnt -= 1;
 	if (siop->sb_refcnt >= 0) {
 /*std_printf("sb_free: refcnt = %d, flag = %d\n", siop->sb_refcnt, flag);*/
 		if (flag) {
@@ -764,16 +765,16 @@ sb_free(n)
 			return;
 		}
 /*std_printf("sb_free: sb_flag = %d\n", siop->sb_flag);*/
-		if (siop->sb_flag != -1)
+		if (siop->sb_flag != (short)-1)
 			return;
 		for (i = 0; i < MAXNFILE ; ++i) {	/* XX: inefficient! */
 /*if (siofds[i] != NULL) std_printf("sb_free: siofds[%d] = %x\n", i, siofds[i]);*/
 			if (i == n || siofds[i] == NULL)
 				continue;
 #if 1
-			if (siofds[i]->_sb_flag == -1 &&
-			    siofds[i]->_sb_refcnt == 0)
-				continue;
+			if (siofds[i]->_sb_flag == (short)-1)
+			  if (siofds[i]->_sb_refcnt == (short)0)
+			    continue;
 #endif
 /*std_printf( "sb_free: siofds[%d]->_sb_data == siop: %d\n", siofds[i]->_sb_data == siop);*/
 			if (siofds[i]->_sb_data == siop)
@@ -783,11 +784,12 @@ sb_free(n)
 		return;
 	}
 	/* we want to free whatever we're pointing to, perhaps us */
-	if (siop->sb_base && (siop->sb_flag & O_CREAT))
-		/* data is malloc'ed */
-		free((char *)siop->sb_base);
+	if (siop->sb_base)
+	  if (siop->sb_flag & O_CREAT)
+	    /* data is malloc'ed */
+	    free((char *)siop->sb_base);
 	if (flag)	/* we're pointing at another siobuf */
-		free((char *)siop->_sb_data);
+	  free((char *)siop->_sb_data);
 /*fprintf(runiofp, "sb_free: done\n");*/
 	free((char *)siop);/* can do this because nothing refers to us */
 }
@@ -984,9 +986,9 @@ runio(ioopp)
 		case sIOdup:
 /*putc('\n', runiofp);*/
 			fd = dup2(ioop->fd, ioop->fd2);
-			if (siofds[ioop->fd2] != NULL &&
-			    siofds[ioop->fd2]->sb_refcnt == 0)
-				sb_free(ioop->fd2)/*, sb_pr()*/;
+			if (siofds[ioop->fd2] != NULL)
+			  if (siofds[ioop->fd2]->sb_refcnt == 0)
+			    sb_free(ioop->fd2)/*, sb_pr()*/;
 			sb_push(&siofds[ioop->fd2], siofds[ioop->fd]);
 /*std_printf( "PUSH(%d) = %x %x (%d)\n", ioop->fd2, siofds[ioop->fd2], siofds[ioop->fd], __LINE__);*/
 			if (isset('R'))
