@@ -543,7 +543,7 @@ rflag:		switch (*++fmt) {
 				 */
 				char *p, *mymemchr();
 
-				if (p = mymemchr(t, 0, prec)) {
+				if ((p = mymemchr(t, 0, prec))) {
 					size = p - t;
 					if (size > prec)
 						size = prec;
