@@ -1236,14 +1236,17 @@ run_listexpand(avl, il)
 	struct conscell *attributes, *tmp;
 	struct conscell *al, *alp = NULL;
 	char *localpart, *origaddr;
-	int c, n, len, errflag, stuff, cnt;
+	int c, n, len, errflag, stuff;
+	volatile int cnt;
 	char *comment, *erroraddress;
 	u_char *s, *s_end;
+	int privilege = -1;
 	struct siobuf *osiop;
 	FILE *mfp, *fp, f;
 	char buf[4096];
 	int fd2;
 	char *olderrors = errors_to;
+	volatile FILE *errstr = stderr;
 
 	extern struct conscell *s_value;
 
@@ -1260,6 +1263,9 @@ run_listexpand(avl, il)
 	    case 'c':
 	      comment = (char*)cdr(il)->string;
 	      break;
+	    case 'p':
+	      privilege = atoi((char*)cdr(il)->string);
+	      break;
 	    case 'e':
 	      erroraddress = (char*)cdr(il)->string;
 	      break;
@@ -1274,13 +1280,16 @@ run_listexpand(avl, il)
 	  }
 	  il = cddr(il); /* Skip TWO entries at the time! */
 	}
+	if (privilege < 0)
+	  privilege = nobody;
+
 	cnt = 0;
 	tmp = il;
 	for (; tmp != NULL; tmp = cdr(tmp)) ++cnt; /* Count arguments.. */
 
 	if (errflag || cnt != 3) {
 		fprintf(stderr,
-			"Usage: %s [ -e error-address ] [ -E errors-to-address ] [ -c comment ] $attribute $localpart $origaddr < /file/path \n",
+			"Usage: %s [ -e error-address ] [ -E errors-to-address ] [-p privilege] [ -c comment ] $attribute $localpart $origaddr < /file/path \n",
 			car(avl)->string);
 		if (errors_to != olderrors)
 		  free(errors_to);
@@ -1435,7 +1444,7 @@ run_listexpand(avl, il)
 	for (ap = hs.h_contents.a; ap != NULL; ap = ap->a_next) {
 		char *av[5];
 		int rc;
-		struct conscell *l, *d;
+		struct conscell *l, *d, *lrc;
 
 		buf[0] = 0;
 		pureAddressBuf(buf,sizeof(buf),ap->a_tokens);
@@ -1444,6 +1453,8 @@ run_listexpand(avl, il)
 
 #define cdddr(x) cdr(cddr(x))
 
+#define use_lapply 1
+#ifdef use_lapply
 		/* XX: We leak these at the moment! */
 		l       = newstring((u_char*)strsave(buf));
 		cdr(l)  = conststring((u_char*)origaddr);
@@ -1454,11 +1465,16 @@ run_listexpand(avl, il)
 		v_set(DEFER, "");
 
 		rc = l_apply("rrouter", l);
-
+		lrc = s_value;
+		s_value = NULL;
+#else
+		lrc = router(ap, privilege, "recipient");
+		rc = (lrc == NULL);
+#endif
 #if 0 /* XX: This HOLD handling didn't work ?? */
 		if (deferit && (d = v_find(DEFER))) {
-		  s_free_tree(s_value);
-		  s_value = NULL;
+		  s_free_tree(lrc);
+		  lrc = NULL;
 		  l = conststring((u_char *)"hold");
 		  cdr(l)   = copycell(cdr(d));
 		  cddr(l)  = newstring((u_char *)strsave(buf));
@@ -1468,10 +1484,12 @@ run_listexpand(avl, il)
 		} else
 		  ;
 #endif
-		if (rc != 0 || s_value == NULL || !LIST(s_value)) {
+		if (rc != 0 || lrc == NULL || !LIST(lrc)) {
 		  /* $(rrouter xx xx xx)  returned something invalid.. */
-		  s_free_tree(s_value);
-		  s_value = NULL;
+#ifdef use_lapply
+		  s_free_tree(lrc);
+#endif
+		  lrc = NULL;
 		  continue;
 		} else {
 		  /*
@@ -1479,32 +1497,36 @@ run_listexpand(avl, il)
 		   * (local - user attributes) or (((local - user attributes)))
 		   * or ( ((local - user attr)) ((local - another attr)) )
 		   */
-		  if (car(s_value) == NULL) {
+		  if (car(lrc) == NULL) {
 		    /* duplicate removal trapped it. Empty list! */
 		    continue;
 		  }
-		  if (LIST(car(s_value))) {
-		    if (!LIST(caar(s_value)) || !STRING(caaar(s_value))) {
+		  if (LIST(car(lrc))) {
+		    if (!LIST(caar(lrc)) || !STRING(caaar(lrc))) {
 		      fprintf(stderr,
 			      "%s: '%s' returned invalid 2-level list: ",
 			      progname, ROUTER);
-		      s_grind(s_value, stderr);
-		      s_free_tree(s_value);
-		      s_value = NULL;
+		      s_grind(lrc, stderr);
+#ifdef use_lapply
+		      s_free_tree(lrc);
+#endif
+		      lrc = NULL;
 		      if (errors_to != olderrors)
 			free(errors_to);
 		      errors_to = olderrors;
 		      return NULL;
 		    }
-		    l = s_copy_tree(s_value);
+		    l = s_copy_tree(lrc);
 		  } else {
-		    l = s_copy_tree(s_value);
+		    l = s_copy_tree(lrc);
 		    l = ncons(l);
 		    l = ncons(l);
 		  }
 
-		  s_free_tree(s_value);
-		  s_value = NULL;
+#ifdef use_lapply
+		  s_free_tree(lrc);
+#endif
+		  lrc = NULL;
 		}
 
 		/* Now the  "(struct conscell *) l" contains a route,
@@ -1537,8 +1559,6 @@ run_listexpand(avl, il)
 		   fwrite(buf,1,len+1,stdout); */
 	}
 
-	&cnt; /* For debugging.. this keeps it in memory with GCC */
-
 	if (al == NULL) { /* ERROR! NO ADDRESSES! */
 	  al = conststring((u_char *)"error");
 	  cdr(al)  = conststring((u_char *)"expansion");
