@@ -52,7 +52,8 @@ search_gdbm(sip)
 	datum val, key;
 	struct conscell *tmp;
 	struct spblk *spl;
-	int retry, symid;
+	int retry;
+	spkey_t symid;
 
 	if (sip->file == NULL)
 		return NULL;
@@ -101,7 +102,7 @@ close_gdbm(sip)
 {
 	GDBM_FILE db;
 	struct spblk *spl;
-	int symid;
+	spkey_t symid;
 
 	if (sip->file == NULL)
 		return;
@@ -125,7 +126,7 @@ open_gdbm(sip, flag, comment)
 {
 	GDBM_FILE db;
 	struct spblk *spl;
-	int symid;
+	spkey_t symid;
 
 	if (sip->file == NULL)
 		return NULL;
@@ -274,7 +275,8 @@ modp_gdbm(sip)
 	int	fno;
 	struct stat stbuf;
 	struct spblk *spl;
-	int symid, rval;
+	spkey_t symid;
+	int rval;
 
 	if (sip->file == NULL
 	    || (db = open_gdbm(sip, O_RDONLY, "modp_gdbm")) == NULL)
