@@ -29,12 +29,6 @@ struct file_map {
 	int	*offsets;	/* Array of line begin offsets	*/
 };
 
-#ifndef	MALLOC_TRACE
-extern unsigned int symbol();
-#endif	/* !MALLOC_TRACE */
-
-
-
 #ifndef	__
 #ifdef __STDC__
 #define __(x) x
@@ -46,6 +40,10 @@ extern unsigned int symbol();
 #endif
 
 
+#ifndef	MALLOC_TRACE
+extern spkey_t symbol __((u_char *s));
+#endif	/* !MALLOC_TRACE */
+
 /* lookups */
 extern struct conscell	*search_bin	__((struct search_info	*sip));
 extern struct conscell	*search_core	__((struct search_info	*sip));
