@@ -47,6 +47,7 @@ extern int hdr_status();
 extern int efstat();
 extern int elink();
 extern int eunlink();
+extern int erename();
 extern int getline();
 extern int nobody;
 extern int s_equal();
@@ -159,8 +160,6 @@ if (fileno(e->e_fp) < 4) {
 
 	if (D_final > 0)
 		dumpInfo(e);
-	if (status != PERR_CTRLFILE && status != PERR_DEFERRED && !savefile)
-		eunlink(file);
 	if (e->e_fp != NULL)
 		fclose(e->e_fp);
 	tfree(MEM_TEMP);
@@ -459,6 +458,11 @@ squirrel(e, text)
 	path = alloca(5+strlen(POSTMANDIR)+10);
 #endif
 	sprintf(path, "../%s/%d", POSTMANDIR, e->e_statbuf.st_ino);
+
+	/* XX: We make a copy by link()ing file to two dirs, some
+	       systems (Andrew-FS in mind) can't do it! Copying is
+	       ok in their case.. */
+
 	if (elink(e->e_file, path) < 0) {
 		sprintf(path, "..%d", e->e_statbuf.st_ino);
 		elink(e->e_file, path);
@@ -625,6 +629,7 @@ reject(e, msgfile)
 		fclose(fp);
 		mail_abort(mfp);
 	}
+	eunlink(e->e_file);
 }
 
 /* Save a message file away because something went wrong during routing */
@@ -648,6 +653,10 @@ defer(e)
 #endif
 	sprintf(path, "../%s/%s", DEFERREDDIR, e->e_file);
 	unlink(path);
+
+	/* XX: Deferrance uses  link()/unlink(), while rename() is equally
+	       valid choice */
+
 	/* try linking every few seconds for a minute */
 	for (i = 0; elink(e->e_file, path) < 0 && i < 10 && errno != ENOENT;++i)
 		sleep(6);
@@ -1026,8 +1035,11 @@ sequencer(e, file)
 
 	errors_to = NULL; /* to be gotten rid off.. */
 
-	if (e == NULL)
+	if (e == NULL) {
+		/* No envelope ??? */
 		return PERR_OK;
+	}
+
 	dprintf("Parse envelope and message header\n");
 	e->e_messageid = NULL;
 	perr = 0;
@@ -1099,8 +1111,10 @@ sequencer(e, file)
 
 	/* put pertinent message information in the zsh environment */
 	setenvinfo(e);
-	if (perr)
+	if (perr) {
+		eunlink(file);
 		return perr;
+	}
 
 	FindEnvelope(eEnvid);
 	envid = NULL;
@@ -1474,8 +1488,10 @@ sequencer(e, file)
 		sender = caar(sender);
 #endif
 
-	if (routed_addresses == NULL)	/* they were probably all deferred */
+	if (routed_addresses == NULL)	/* they were probably all deferred */ {
+		printf("No routed addresses -> deferred\n");
 		return PERR_DEFERRED;
+	}
 
 	rwhead = NULL;
 	for (idnumber = 0, l = car(routed_addresses); l != NULL ; l = cdr(l)) {
@@ -1606,6 +1622,7 @@ sequencer(e, file)
 #ifndef USE_ALLOCA
 		free(ofpname);
 #endif
+		printf("Creation of control file failed\n");
 		return PERR_CTRLFILE;
 	}
 	setvbuf(ofp, vbuf, _IOFBF, sizeof vbuf);
@@ -1819,9 +1836,7 @@ sequencer(e, file)
 		fprintf(vfp, "router done processing %s\n", file);
 		fclose(vfp);
 	}
-
-	/* fflush(ofp);
-	   fsync(fileno(ofp)); */
+	
 	if (e->e_fp != NULL && files_gid >= 0) {
 		fchown(fileno(e->e_fp), -1, files_gid);
 		fchmod(fileno(e->e_fp),
@@ -1834,7 +1849,7 @@ sequencer(e, file)
 	qpath = emalloc(5+strlen(QUEUEDIR)+strlen(file));
 #endif
 	sprintf(qpath, "../%s/%s", QUEUEDIR, file);
-	if ((fclose(ofp) == EOF) || (elink(file, qpath) < 0)) {
+	if ((fclose(ofp) == EOF) || (erename(file, qpath) < 0)) {
 		unlink(qpath);
 		unlink(ofpname);
 #ifndef	USE_ALLOCA
@@ -1843,32 +1858,8 @@ sequencer(e, file)
 #endif
 		return PERR_CTRLFILE;
 	}
-	/*
-	 * link the control file to the scheduler and transport dirs,
-	 * in the expectation the scheduler will unlink it from the
-	 * scheduler directory once it has been assimilated.  Note
-	 * that the order matters if one wants to avoid a scheduler
-	 * race condition on its startup.
-	 */
-#ifdef	USE_ALLOCA
-	path = alloca(5+strlen(SCHEDULERDIR)+strlen(file));
-#else
-	path = emalloc(5+strlen(SCHEDULERDIR)+strlen(file));
-#endif
-	sprintf(path, "../%s/%s", SCHEDULERDIR, file);
-	unlink(path);
-	if (elink(ofpname, path) < 0) {
-		unlink(qpath);
-		unlink(path);
-#ifndef	USE_ALLOCA
-		free(ofpname);
-		free(qpath);
-		free(path);
-#endif
-		return PERR_CTRLFILE;
-	}
-#ifndef	USE_ALLOCA
-	free(path);
+
+#ifndef USE_ALLOCA
 	path = emalloc(5+strlen(TRANSPORTDIR)+strlen(file));
 #else
 	/* This actually reallocs more space from stack, but then it
@@ -1876,8 +1867,8 @@ sequencer(e, file)
 	path = alloca(5+strlen(TRANSPORTDIR)+strlen(file));
 #endif
 	sprintf(path, "../%s/%s", TRANSPORTDIR, file);
-	unlink(path);
-	if (elink(ofpname, path) < 0) {
+	unlink(path);	/* Should actually always fail.. */
+	if (erename(ofpname, path) < 0) {
 		unlink(qpath);
 		unlink(path);
 #ifndef	USE_ALLOCA
@@ -1888,8 +1879,6 @@ sequencer(e, file)
 		return PERR_CTRLFILE;
 	}
 
-	eunlink(ofpname);
-
 #ifndef	USE_ALLOCA
 	free(ofpname);
 	free(qpath);
