@@ -10,7 +10,6 @@
 extern int gotalarm;
 extern int numkids;
 extern int ranny();
-extern int sweepinterval;
 extern int verbose;
 extern time_t time();
 extern void expire();
@@ -18,16 +17,19 @@ extern void fifoput();
 extern void queryipcinit();
 extern void spool();
 extern void transport();
-extern void unravel();
+extern int  unravel();
 
+extern int sweepinterval;
 
 struct vertex *curitem = NULL;
 
 time_t qipcretry = 0;
-time_t sweepretry = 0;
 
 #define SALARM(N)  (verbose ? printf("alarm(%d) = %d\n", (N),alarm(N)):alarm(N))
 
+/*
+ * Detach the vertex from its chains
+ */
 void
 disentangle(vp)
 	struct vertex *vp;
@@ -177,28 +179,32 @@ reschedule(vp, factor, index)
 	  vp->nextitem = curitem;
 	  vp->previtem = NULL;
 	  curitem = vp;
-	  if (curitem->wakeup - now > 0
-	      && curitem->wakeup - now <= sweepinterval)
+	  if (curitem->wakeup > now  &&
+	      (curitem->wakeup + sweepinterval) <= now)
 	    SALARM((u_int)(curitem->wakeup - now));
 	  else
 	    SALARM(sweepinterval/2);
 	}
 }
 
-void
+int	/* return non-zero when there are some childs
+	   available for start				*/
 doagenda()
 {
 	time_t now;
 	int didsomething;
 	struct vertex *vp, *ncuritem;
+	int no_more = 0;
+
+	time(&now);
 
 	ncuritem = curitem;
 	curitem = NULL;
+	didsomething = 0;
 	do {
-	  now = time((time_t *)0);
 	  didsomething = 0;
 	  if (verbose)
-	    printf("curitem %x curitem->wakeup %d now %d\n",
+	    printf("curitem %x curitem->wakeup %lu now %lu\n",
 			  ncuritem,
 			  ncuritem != NULL ? ncuritem->wakeup : 0,
 			  now);
@@ -211,13 +217,15 @@ doagenda()
 	      vp->previtem->nextitem = NULL;
 	      vp->previtem = NULL;
 	    }
-	    unravel(ncuritem);
+	    no_more = unravel(ncuritem);
 	    ncuritem = vp;
+	    if (no_more)
+	      break;
 	  }
 
-	  mux(1); /* Fast check */
+	  mux(0); /* Rather fast check.. */
+	} while (didsomething && !no_more);
 
-	} while (didsomething);
 	for (vp = curitem ; vp != NULL ; vp = vp->nextitem)
 	  if (vp->nextitem == NULL)
 	    break;
@@ -228,8 +236,12 @@ doagenda()
 	  }
 	} else
 	  curitem = ncuritem;
-	if (verbose)
-	  printf("alarmed %d\n", now);
+
+	time(&now);
+
+	/* if (verbose)
+	   printf("alarmed %d\n", now);  */
+
 	if (qipcretry > 0 && qipcretry <= now) {
 	  qipcretry = 0;
 	  queryipcinit();
@@ -238,16 +250,10 @@ doagenda()
 	   * that's ok since sweepretry is active by now
 	   */
 	}
-	if (sweepretry > 0 && sweepretry <= now)
-	  sweepretry = now + sweepinterval;
-	if (ncuritem != NULL && ncuritem->wakeup < sweepretry)
-	  SALARM((u_int)(ncuritem->wakeup - now));
-	else if (sweepretry > now)
-	  SALARM(sweepretry - now);
-	gotalarm = 0;
+	return (didsomething && !no_more);
 }
 
-void
+int	/* Return non-zero, when can not start more children */
 unravel(head)
 	struct vertex *head;
 {
@@ -381,10 +387,6 @@ spool(thread)
 	     *   (Anything goes, as long as it is  != V_NONE)
 	     */
 	    vp->cfp->mark = V_SELECT;
-if (verbose && vp->cfp->atspool != NULL)
-  printf("spool(): vp->cfp->atspool != NULL ! (-> \"%s\")\n",
-	 vp->cfp->atspool->linkpath);
-	    vp->cfp->atspool = vp;
 	  }
 	}
 	transport(thread, thread->orig[L_CHANNEL]->name,
