@@ -78,17 +78,15 @@ msgerror(vp, offset, message)
 	long offset;
 	char *message;
 {
-	char path[MAXPATHLEN];
 	FILE *fp;
 	char *notary = "";
 
 	if (vp->notary) notary = vp->notary;
 
-	sprintf(path, "../%s/%s", TRANSPORTDIR, vp->cfp->mid);
 	/* exclusive access required, but we're the only scheduler... */
-	if ((fp = fopen(path, "a")) == NULL) {
+	if ((fp = fopen(vp->cfp->mid, "a")) == NULL) {
 	  fprintf(stderr,
-		  "Cannot open control file %s to deposit", path);
+		  "Cannot open control file %s to deposit", vp->cfp->mid);
 	  fprintf(stderr,
 		  " error message for offset %ld:\n", offset);
 	  fprintf(stderr, "\t%s\n", message);
@@ -130,13 +128,13 @@ reporterrs(cfpi)
 	envid = cfpi->envid;
 	/* re-read the control file to get some unstashed information */
 	midbuf = cfpi->mid;
-	sprintf(path, "../%s/%s", TRANSPORTDIR, cfpi->mid);
 
 	/* exclusive access required, but we're the only scheduler... */
-	if ((fd = open(path, O_RDWR, 0)) < 0 || (cfp = slurp(fd)) == NULL) {
+	if ((fd = open(cfpi->mid, O_RDWR, 0)) < 0 ||
+	    (cfp = slurp(fd)) == NULL) {
 	  fprintf(stderr,
 		  "%s: unexpected absence of control file %s for error processing!\n",
-		  progname, path);
+		  progname, cfpi->mid);
 	  if (fd >= 0)
 	    close(fd);
 	  return;
