@@ -62,7 +62,8 @@ int	subsubdirs;	/* use subdirectories under each channel directory */
 int	D_alloc = 0;
 int	vtxprep_skip = 0;
 int	vtxprep_skip_any = 0;
-time_t	next_delaysweep;
+int	vtxprep_skip_lock = 0;
+time_t	next_dirscan	= 0;
 
 #include "memtypes.h"
 extern memtypes stickymem;
@@ -99,27 +100,38 @@ extern struct config_entry *readconfig __((char *configfile));
 extern struct config_entry *rereadconfig __((struct config_entry *,char *));
 extern struct ctlfile *schedule __((int fd, char *file, struct config_entry *ce));
 extern struct ctlfile *slurp __((int fd)), *vtxprep __((struct ctlfile *));
-extern time_t sweepretry;
 extern time_t time();
 extern u_char *rfc822date __((time_t *));
 extern void deletemsg __((char *, struct ctlfile *));
 extern void detach __((void));
 extern void die __((int rc, char *reason));
-extern void doagenda __((void));
+extern int  doagenda __((void));
 extern void killprevious __((int signal, char *pidfile));
 extern void link_in __((int flag, struct vertex *vp, u_char *s));
 extern void mklink __((char *from, char *to));
-extern void mux __((int quickcheck));
+extern int  mux __((time_t timeout));
 extern void prversion __((char *name));
 extern void qprint __((int fd));
 extern void queryipcinit __((void));
 extern void reschedule __((struct vertex *vp, int factor, int index));
-extern int  syncweb __((char *dir, int nukefile, struct config_entry *ce));
 extern void unctlfile __((struct ctlfile *cfp));
 extern void vtxdo __((struct vertex *vp, struct config_entry *ce, char *path));
 extern void vtxprint __((struct vertex *vp));
 extern void defaultconfigentry __((struct config_entry *ce, struct config_entry *));
 
+struct dirstatname {
+	struct stat st;
+	char name[1]; /* Allocate enough size */
+};
+struct dirqueue {
+	int	wrkcount;
+	int	wrkspace;
+	struct dirstatname **stats;
+};
+
+extern int dirqueuescan __((const char *dir, struct dirqueue *dq));
+extern int  syncweb __((struct dirqueue *dq, struct config_entry *ce));
+
 int global_maxkids = 1000;
 
 int
@@ -133,9 +145,13 @@ main(argc, argv)
 	long offout, offerr;
 	time_t now;
 	struct config_entry *tp;
+	struct dirqueue dqb;
+	struct dirqueue *dq = &dqb;
 
 	/* setlinebuf(stderr);  -- no need for this ? */
 
+	memset(&dqb,0,sizeof(dqb));
+
 	if ((progname = strrchr(argv[0], '/')) == NULL)
 		progname = argv[0];
 	else
@@ -261,9 +277,11 @@ main(argc, argv)
 
 	if (postoffice == NULL && (postoffice = getzenv("POSTOFFICE")) == NULL)
 	  postoffice = POSTOFFICE;
-	if (chdir(postoffice) < 0 || chdir(SCHEDULERDIR) < 0)
+
+	if (chdir(postoffice) < 0 || chdir(TRANSPORTDIR) < 0)
 	  fprintf(stderr, "%s: cannot chdir to %s/%s.\n",
-		  progname, postoffice, SCHEDULERDIR);
+		  progname, postoffice, TRANSPORTDIR);
+
 	if (rendezvous == NULL && (rendezvous = getzenv("RENDEZVOUS")) == NULL)
 	  rendezvous = qoutputfile;
 	if (daemonflg) {
@@ -316,53 +334,56 @@ main(argc, argv)
 	SIGNAL_IGNORE(SIGPIPE);
 	SIGNAL_HANDLE(SIGALRM, sig_alarm);	/* process agenda */
 	SIGNAL_HANDLE(SIGUSR2, sig_iot);	/* dump queue info */
-	cp = emalloc(strlen(TRANSPORTDIR)+4);
-	sprintf(cp, "../%s", TRANSPORTDIR);
+
 	queryipcinit();
-	vtxprep_skip_any = 0;
-	syncweb(cp, 0, tp);
-	if (!vtxprep_skip_any) {
-	  free(cp);
-	  cp = NULL;
-	}
+
+
+	dirqueuescan(".", dq);
+
+	vtxprep_skip_lock = 0;
+	syncweb(dq, tp);
+
 	if (dlyverbose) verbose = dlyverbose;
-	time(&next_delaysweep);
-	next_delaysweep += 1*60; /* One minute */
 
 	canexit = 1;
 	SIGNAL_HANDLE(SIGTERM, sig_exit);	/* split */
-	sweepretry = 1;
-	alarm(sweepinterval);
 #ifdef	MALLOC_TRACE
 	mal_leaktrace(1);
 #endif	/* MALLOC_TRACE */
 	while (!mustexit) {
 	  int wrkcnt;
+	  time_t now, timeout;
+
+	  time(&now);
+
 	  canexit = 0;
-	  if (cp != NULL &&  time(NULL) > next_delaysweep) {
-	    /* Delayed jobs in queue.. */
-	    vtxprep_skip_any = 0;
-	    fprintf(stderr,"Delay-sweeping previously locked files\n");
-	    wrkcnt = syncweb(cp, 0, tp);
-	    if (!vtxprep_skip_any) {
-	      fprintf(stderr," ... all done!\n");
-	      free(cp);
-	      cp = NULL;
-	    } else
-	      fprintf(stderr,
-		      " ... scheduled %d files successfully, %d skipped.\n",
-		      wrkcnt, vtxprep_skip_any);
-	    time(&next_delaysweep);
-	    next_delaysweep += 5*60; /* Five minutes */
+
+	  if (now >= next_dirscan) {
+	    /* Directory scan time for new jobs .. */
+	    next_dirscan = now + sweepinterval;	/* 10 seconds interval */
+	    dirqueuescan(".", dq);
 	  }
-	  syncweb(".", 1, tp);
-	  doagenda();
-	  /*
-	   * Due to a possible race condition, gotalarm = 0 must
-	   * have been done inside doagenda() instead of here.
-	   */
+
+	  /* Submit one item from pre-scheduler-queue into scheduler */
+	  if (dq->wrkcount > 0)
+	    syncweb(dq, tp);
+
+	  /* See when to timeout from mux() */
+	  timeout = next_dirscan;
+
+	  /* If we still have things in pre-scheduler queue... */
+	  if (dq->wrkcount > 0)
+	    timeout = now;
+
+	  /* Submit possible new jobs */
+	  if (doagenda() != 0)
+	    timeout = now;	/* we still have some jobs avail for start */
+
+	  gotalarm = 0;
+	  canexit = 1;
+
 	  /* mux on the stdout of children */
-	  while (!gotalarm && numkids > 0) {
+	  do {
 	    if (dumpq) {
 	      qprint(-1);
 	      dumpq = 0;
@@ -371,21 +392,8 @@ main(argc, argv)
 	      tp = rereadconfig(tp, config);
 	      rereadcf = 0;
 	    }
-	    mux(0);		/* Slow check */
-	  }
-	  canexit = 1;		/* may be possible to move this after doagenda() */
-	  /* wait for an alarm call */
-	  while (!gotalarm) {
-	    if (dumpq) {
-	      qprint(-1);
-	      dumpq = 0;
-	    }
-	    if (rereadcf) {
-	      tp = rereadconfig(tp, config);
-	      rereadcf = 0;
-	    }
-	    mux(0);		/* Slow check */
-	  }
+	  } while (mux(timeout) == 0 && !gotalarm &&
+		   dq->wrkcount == 0);
 	}
 #ifdef	MALLOC_TRACE
 	mal_dumpleaktrace(stderr);
@@ -443,80 +451,130 @@ int sig;
  * Absorb any new files that showed up in our directory.
  */
 
+static int
+in_dirscanqueue(dq,ino)
+	struct dirqueue *dq;
+	long ino;
+{
+	int i;
+
+	/* Return 1, if can find the "ino" in the queue */
+
+	if (dq->wrkcount == 0) return 0;
+	for (i=0; i < dq->wrkcount; ++i)
+	  if (ino == dq->stats[i]->st.st_ino)
+	    return 1;
+
+	return 0;
+}
+
+static int
+dq_ctimecompare(a,b)
+const void *a;
+const void *b;
+{
+	const struct dirstatname **A = (const struct dirstatname **)a;
+	const struct dirstatname **B = (const struct dirstatname **)b;
+
+	return ((*A)->st.st_ctime - (*B)->st.st_ctime);
+}
+
 int
-syncweb(dir, nukefile, cehead)
-	char *dir;
-	int nukefile;
-	struct config_entry *cehead;
+dirqueuescan(dir, dq)
+	const char *dir;
+	struct dirqueue *dq;
 {
-	struct stat stbuf;
-	int fd;
-	char file[MAXNAMLEN+1];
 	DIR *dirp;
 	struct NDIR_TYPE *dp;
-	static time_t modtime = 0;
-	int wrkcnt = 0;
+	struct stat stbuf;
+	char file[MAXNAMLEN+1];
+	int newents = 0;
+	struct dirstatname *dsn;
 
+#if 0
+	static time_t modtime = 0;
+
+	/* Any changes lately ? */
 	if (estat(dir, &stbuf) < 0)
-	  return -1;
+	  return -1;	/* Could not stat.. */
 	if (stbuf.st_mtime == modtime)
-	  return 0;
+	  return 0;	/* any changes lately ? */
 	modtime = stbuf.st_mtime;
+#endif
+
+	/* Some changes lately, open the dir and read it */
+
 	dirp = opendir(dir);
 	for (dp = readdir(dirp); dp != NULL; dp = readdir(dirp)) {
-	  if (mustexit)
-	    break;
+	  /* Scan filenames into memory */
+	  if (dp->d_name[0] >= '0' &&
+	      dp->d_name[0] <= '9') {
+	    /* A file whose name STARTS with a number (digit) */
+	    strcpy(file, dp->d_name);
 
-	  /* mux(1); /* Fast check */
-
-	  if (isascii(dp->d_name[0]) && isdigit(dp->d_name[0])) {
-	    if (strcmp(dir, ".") == 0)
-	      strcpy(file, dp->d_name);
-	    else
-	      sprintf(file, "%s/%s", dir, dp->d_name);
-	    if ((fd = eopen(file, O_RDWR, 0)) < 0) {
-	      if (getuid() == 0)
-		eunlink(file);	/* hrmpf! */
-	    } else {
-	      if (!nukefile) {
-		/* If we are re-syncing in from the $POSTOFFICE/transports/
-		   directory, we may have this file in processing state...  */
-		struct spblk *spl;
-		fstat(fd,&stbuf);
-		spl = sp_lookup((long)stbuf.st_ino, spt_mesh[L_CTLFILE]);
-		if (spl != NULL) {
-		  /* Already in processing, don't touch.. */
-		  /*printf("File: %s active (not previously locked)\n",file);*/
-		  close(fd);
-		  ++vtxprep_skip_any;
-		  continue;
-		}
-		/* Ok, perhaps it is not in active processing, sync it in.. */
-	      }
-	      schedule(fd, file, cehead);
-	      ++wrkcnt;
-	      if (nukefile) {
-		unlink(file);
-		if (verbose)
-		  printf("scheduler: completed %s\n",file);
-	      } else {		/* we're scanning TRANSPORTDIR */
-		/* unlink it from the SCHEDULERDIR */
-		unlink(dp->d_name);
-		if (verbose)
-		  printf("scheduler: completed %s\n",dp->d_name);
-	      }
+	    if (lstat(file,&stbuf) != 0 ||
+		!S_ISREG(stbuf.st_mode)) {
+	      /* Let it be there for the manager to wonder.. */
 	      /*
-	       * This avoids a race when the
+	       * This also avoids a race when the
 	       * scheduler starts up while the router
-	       * is pushing stuff at it.  Note that
-	       * there is a window of vulnerability
-	       * if the router links to transport
-	       * then scheduler instead of vice
-	       * versa.
+	       * is pushing stuff at it, and at first
+	       * links to the scheduler, and then to
+	       * the transport dir..
+	       *
+	       * We propably need to check on this (directory)
+	       * latter to see if the file appeared also to the
+	       * transport dir.
 	       */
+	      continue;
 	    }
-	  }
+
+	    if (in_dirscanqueue(dq,stbuf.st_ino))
+	      /* Already in pre-schedule-queue */
+	      continue;
+
+	    /* If we are re-syncing in from the $POSTOFFICE/transports/
+	       directory, we may have this file in processing state...  */
+	    {
+	      struct spblk *spl;
+	      spl = sp_lookup((long)stbuf.st_ino, spt_mesh[L_CTLFILE]);
+	      if (spl != NULL) {
+		/* Already in processing, don't touch.. */
+		/*printf("File: %s active (not previously locked)\n",file);*/
+		++vtxprep_skip_any;
+		continue;
+	      }
+	    }
+
+	    /* Now store the entry */
+	    dsn = (struct dirstatname*)emalloc(sizeof(*dsn)+strlen(file));
+	    memcpy(&(dsn->st),&stbuf,sizeof(stbuf));
+	    strcpy(dsn->name,file);
+
+	    if (dq->wrkspace <= dq->wrkcount) {
+	      /* Increase the space */
+	      dq->wrkspace += 8;
+	      if (dq->stats == NULL)
+		dq->stats = (struct dirstatname **)emalloc(sizeof(void*) *
+							   dq->wrkspace);
+	      else
+		dq->stats = (struct dirstatname**)erealloc(dq->stats,
+							   sizeof(void*) *
+							   dq->wrkspace);
+	    }
+	    dq->stats[dq->wrkcount] = dsn;
+	    dq->wrkcount += 1;
+	    ++newents;
+	  } /* ... end of "filename starts with a [0-9]" */
 	}
+	if (newents) {
+	  /* Sort the dq->stats[] per file ctime -- last on slot 0. */
+	  qsort((void*)dq->stats,
+		dq->wrkcount,
+		sizeof(void*),
+		dq_ctimecompare );
+	}
+
 #ifdef	BUGGY_CLOSEDIR
 	/*
 	 * Major serious bug time here;  some closedir()'s
@@ -527,6 +585,52 @@ syncweb(dir, nukefile, cehead)
 #endif
 	closedir(dirp);
 
+	return newents;
+}
+
+int
+syncweb(dq, cehead)
+	struct dirqueue *dq;
+	struct config_entry *cehead;
+{
+	struct stat *stbuf;
+	char *file;
+	struct spblk *spl;
+	int wrkcnt = 0;
+
+	/* Any work to do ? */
+	if (dq->wrkcount == 0) return 0;
+
+	/* Ok some, decrement the count to change it to index */
+	dq->wrkcount -= 1;
+	file  =   dq->stats[dq->wrkcount]->name;
+	stbuf = &(dq->stats[dq->wrkcount]->st);
+	/* Now we have pointers */
+
+	/* Sometimes we may get files already in processing
+	   into our pre-schedule queue */
+
+	/* Already in processing ? */
+	spl = sp_lookup((long)stbuf->st_ino, spt_mesh[L_CTLFILE]);
+	if (spl == NULL) {
+	  /* Not yet in processing! */
+	  int fd;
+
+	  /* Can open ? */
+	  if ((fd = eopen(file, O_RDWR, 0)) < 0) {
+	    if (getuid() == 0)
+	      eunlink(file);	/* hrmpf! */
+	  } else {
+	    /* Ok, schedule! */
+	    schedule(fd, file, cehead);
+	    ++wrkcnt;
+	  }
+	}
+
+	/* Free the pre-schedule queue entry */
+	free(dq->stats[dq->wrkcount]);
+	dq->stats[dq->wrkcount] = NULL;
+
 	return wrkcnt;
 }
 
@@ -545,7 +649,6 @@ schedule(fd, file, cehead)
 	struct config_entry *cehead;
 {
 	struct ctlfile *cfp;
-	char path[MAXPATHLEN], buf[MAXPATHLEN], *dir;
 	struct vertex *vp;
 
 	/* read and process the control file */
@@ -562,33 +665,8 @@ schedule(fd, file, cehead)
 	  unctlfile(cfp);
 	  return NULL;
 	}
-	/* -- DO NOT FREE CFP->CONTENTS YET! 
-	   if (cfp->contents != NULL && cfp->vfp == NULL) {
-	   free(cfp->contents);
-	   cfp->contents = NULL;
-	   }
-	*/
 	for (vp = cfp->head; vp != NULL; vp = vp->next[L_CTLFILE]) {
-	  if (subsubdirs) {	/* channel/host/file */
-	    dir = vtx_to_dir(vp, buf);
-	  } else if (vp->next[L_CTLFILE] == NULL
-		     || (vp->orig[L_CHANNEL] !=
-			 vp->next[L_CTLFILE]->orig[L_CHANNEL])) {
-	    dir = vp->orig[L_CHANNEL]->name; /* channel/file */
-	  } else
-	    dir = NULL;
-	  *path = 0;
-	  if (dir != NULL) {
-	    sprintf(path, "%s/%lu", dir, cfp->id);
-	    mklink(file, path);
-	    if (cfp->vfp != NULL)
-	      fprintf(cfp->vfp, "linked to %s\n", path);
-	    if (verbose)
-	      printf("linked %s,%ld to %s\n",
-		     vp->cfp->mid, vp->cfp->id, path);
-	  }
-
-	  vtxdo(vp, cehead, path);
+	  vtxdo(vp, cehead, file);
 	  if (cfp->vfp)
 	    fseek(cfp->vfp, 0, 2);
 	}
@@ -698,6 +776,7 @@ struct offsort {
 	int	myidx;
 	long	headeroffset;
 	long	drptoffset;
+	char	*sender;
 };
 
 /* Used by the qsort comparison routine, bcp points to the control file bytes */
@@ -746,6 +825,7 @@ vtxprep(cfp)
 	int svn;
 	char *cp, *channel, *host, *l_channel, *l_host;
 	char *echannel, *ehost, *l_echannel, *l_ehost, mfpath[100], flagstr[2];
+	char *latest_sender = NULL;
 	struct spblk *spl;
 	struct vertex *vp, *pvp, **pvpp, *head;
 	struct stat stbuf;
@@ -774,9 +854,6 @@ vtxprep(cfp)
 	     * for the rename(), we might as well do it here,
 	     * instead of in some separate program.
 	     *
-	     * NB! If transport agents are running, this may
-	     * cause duplicate delivery of a message.  Better than
-	     * no delivery...
 	     */
 	    if (!lockverify(cfp,cp)) {
 	      /*
@@ -796,16 +873,26 @@ vtxprep(cfp)
 	      free(cfp->contents);
 	      free((char *)cfp);
 	      ++vtxprep_skip;
+	      ++vtxprep_skip_lock;
 	      return NULL;
 	    }
 	    *cp = _CFTAG_NORMAL; /* unlock it */
 	    lockaddr(cfp->fd, (long) (cp - cfp->contents),
 		     _CFTAG_LOCK, _CFTAG_NORMAL);
 	  }
-	  if (*cp == _CFTAG_NORMAL
-	      || *cp == '\n' /* This appears for msg-header entries.. */ ) {
+	  if (*cp == _CFTAG_NORMAL ||
+	      *cp == '\n' /* This appears for msg-header entries.. */ ) {
 	    --cp;
 	    switch (*cp) {
+	    case _CF_SENDER:
+	      ++cp;
+	      while (*cp == ' ') ++cp;
+	      while (*cp != 0 && *cp != ' ') ++cp; while (*cp == ' ') ++cp;
+	      while (*cp != 0 && *cp != ' ') ++cp; while (*cp == ' ') ++cp;
+	      latest_sender = cp;
+	      while (*cp != 0 && *cp != ' ') ++cp;  *cp = 0; /* No spaces at the sender addr! */
+	      cfp->erroraddr = latest_sender;
+	      break;
 	    case _CF_RECIPIENT:
 	      if (opcnt >= offspc-1) {
 		offspc *= 2;
@@ -822,6 +909,7 @@ vtxprep(cfp)
 	      offarr[opcnt].myidx = i;
 	      offarr[opcnt].headeroffset = -1;
 	      offarr[opcnt].drptoffset = -1;
+	      offarr[opcnt].sender = latest_sender;
 	      ++opcnt;
 	      break;
 	    case _CF_RCPTNOTARY:
@@ -1033,6 +1121,7 @@ vtxprep(cfp)
 	      vp->ngroup = i - svn;
 	      vp->attempts = 0;
 	      vp->notary = NULL;
+	      vp->sender = offarr[i].sender;
 	      vp->headeroffset = offarr[i].headeroffset; /* Just any of them will do */
 	      vp->drptoffset = offarr[i].drptoffset;
 	      while (svn < i) {
@@ -1077,6 +1166,7 @@ vtxprep(cfp)
 	  vp->ngroup = i - svn;
 	  vp->attempts = 0;
 	  vp->notary = NULL;
+	  vp->sender = offarr[i].sender;
 	  vp->headeroffset = offarr[svn].headeroffset; /* Just any of them will do */
 	  vp->drptoffset = offarr[svn].drptoffset;
 	  while (svn < i) {
@@ -1356,32 +1446,20 @@ file_to_ctl(s)
 	return (struct ctlfile *)spl->data;
 }
 
-void
-mklink(from, to)
-	char *from, *to;
+/*
+ * time-string to be prepended on logged messages
+ */
+char *
+timestring()
 {
-	char *cp;
+	static char timebuf[40];
+	time_t now;
+	struct tm *tp;
 
-	if (link(from, to) >= 0)
-	  return;
-	switch (errno) {
-	case ENOENT:	/* this may mean a subdirectory doesn't exist */
-	  for (cp = strchr(to, '/'); cp != NULL; cp = strchr(cp+1, '/')) {
-	    *cp = '\0';
-	    mkdir(to, 0755);
-	    *cp = '/';
-	  }
-	  if (link(from, to) >= 0)
-	    return;
-	  /* fall through (down to default action) */
-	case EEXIST:	/* ignore this, since file may exist previously */
-	  if (errno == EEXIST)
-	    return;
-	  /* fall through (for the sake of second attempt in ENOENT) */
-	default:	/* serious problem of some kind */
-	  fprintf(stderr, "%s: link(%s, %s): %s\n", progname,
-		  from, to, strerror(errno));
-	  break;
-	}
-	return;
+	time(&now);
+	tp = localtime(&now);
+	sprintf(timebuf,"%d%02d%02d%02d%02d%02d",
+		tp->tm_year+1900, tp->tm_mon+1, tp->tm_mday,
+		tp->tm_hour, tp->tm_min, tp->tm_sec);
+	return timebuf;
 }
