@@ -30,11 +30,14 @@ extern void reporterrs();
 extern void reschedule();
 
 extern int verbose;
+extern struct procinfo *cpids;
 
 /*
  * Parse a diagnostic string from a transport agent, and do whatever is
  * necessary to update our model of the world to reflect the new reality.
  *
+ *	#hungry
+ *	# other debug comment
  *	18453/3527\tIETF-NOTARY\tok Quick SMTP connection!
  *	18453/3527\tIETF-NOTARY\tdeferred Unable to contact host!
  *	18453/3527\tIETF-NOTARY\terror Unknown user name "fred".
@@ -57,7 +60,8 @@ static struct diagcodes {
 };
 
 void
-update(diagnostic)
+update(fd, diagnostic)
+	int fd;
 	char	*diagnostic;
 {
 	register char	*cp;
@@ -66,7 +70,16 @@ update(diagnostic)
 	long	inum;
 	struct diagcodes *dcp;
 
+	if (*diagnostic == 0) {
+	  /* Lone newline.. */
+	  cpids[fd].hungry = 1;
+	  return;
+	}
 	if (*diagnostic == '#' /* DEBUG comment.. */) {
+	  if (strncmp(diagnostic,"#hungry",7)==0) {
+	    cpids[fd].hungry = 1;
+	    return;
+	  }
 	  fprintf(stderr, "DBGdiag: %s\n", diagnostic);
 	  return;
 	}
@@ -144,15 +157,14 @@ unctlfile(cfp)
 #ifdef	LOG_INFO
 	syslog(LOG_INFO, "%s: done", cfp->mid);
 #endif	/* LOG_INFO */
-	sprintf(path, "../%s/%s", TRANSPORTDIR, cfp->mid);
-	eunlink(path);
-if (verbose)
-  printf("%s: unlink %s", cfp->logident, path);
+	eunlink(cfp->mid);
+	if (verbose)
+	  printf("%s: unlink %s", cfp->logident, cfp->mid);
 
 	sprintf(path, "../%s/%s", QUEUEDIR, cfp->mid);
 	eunlink(path);
-if (verbose)
-  printf("   and %s/\n", path);
+	if (verbose)
+	  printf("   and %s/\n", path);
 
 	if (cfp->vfp != NULL) {
 	  fseek(cfp->vfp, 0, 2);
@@ -178,73 +190,7 @@ unvertex(vp)
 
 	if (vp->ngroup > 0)
 	  return;
-	/*
-	 * If we remove a vertex while using channel/host/controlfile, we
-	 * can unlink that file right here & now.
-	 */
-	if (subsubdirs) {
-	  sprintf(path, "%s/%s/%lu",
-		  vp->orig[L_CHANNEL]->name,
-		  vp->orig[L_HOST]->name,
-		  vp->cfp->id);
-	  eunlink(path);
 
-if (verbose)
-  printf("%s: unlink %s\n", vp->cfp->logident, path);
-
-	  if (vp->cfp->vfp != NULL) {
-	    fseek(vp->cfp->vfp, 0, 2);
-	    fprintf(vp->cfp->vfp, "unlinked %s\n", path);
-	    /*printf("%s: unlink %s\n", vp->cfp->logident, path);*/
-	  }
-	  /*
-	   * If this is the last channel/host combination for this
-	   * control file, remove it. (see large comment below).
-	   */
-	  for (vv = vp->cfp->head; vv != NULL; vv = vv->next[L_CTLFILE]) {
-	    if (vv != vp
-		&& vv->orig[L_CHANNEL] == vp->orig[L_CHANNEL]
-		&& vv->orig[L_HOST] == vp->orig[L_HOST])
-	      break;
-	  }
-	  if (vv == NULL) {
-	    if ((cp = strrchr(path, '/')) != NULL) /* assert! */
-	      *cp = '\0';
-	    rmdir(path);	/* the directory */
-	    /* printf("%s: rmdir %s\n", vp->cfp->logident, path);*/
-	  }
-	} else {
-	  /*
-	   * If we use channel/controlfile, we can only unlink()
-	   * the controlfile if there are no more vertices for this
-	   * channel, belonging to this control file.
-	   * Instead of getting into a long complicated boolean
-	   * expression, we apply the KISS principle (as above).
-	   * Unfortunately this makes the whole thing O(n^2)...
-	   * Many optimizations are possible, but not obviously
-	   * worthwhile.
-	   */
-	  for (vv = vp->cfp->head; vv != NULL; vv = vv->next[L_CTLFILE]) {
-	    if (vv != vp
-		&& vv->orig[L_CHANNEL] == vp->orig[L_CHANNEL])
-	      break;
-	  }
-	  if (vv == NULL) {
-	    sprintf(path, "%s/%lu",
-		    vp->orig[L_CHANNEL]->name,
-		    vp->cfp->id);
-	    eunlink(path);
-if (verbose)
-  printf("%s: unlink %s\n", vp->cfp->logident, path);
-
-	    if (vp->cfp->vfp != NULL) {
-	      fseek(vp->cfp->vfp, 0, 2);
-	      fprintf(vp->cfp->vfp, "unlinked %s\n", path);
-	      /* printf("%s: unlink %s\n", vp->cfp->logident,
-		 path); */
-	    }
-	  }
-	}
 	for (i = 0; i < SIZE_L; ++i) {
 	  if (vp->next[i] != NULL)
 	    vp->next[i]->prev[i] = vp->prev[i];
@@ -260,16 +206,6 @@ if (verbose)
 	    if ((vp->orig[i]->lastlink = vp->prev[i]) == NULL)
 	      remove = 1;
 	  if (remove) {
-	    if (i == L_CHANNEL) {
-	      /*
-	       * If we are removing a channel, obviously
-	       * the directory is empty...
-	       */
-	      sprintf(path, "%s", vp->orig[L_CHANNEL]->name);
-	      ermdir(path);
-	      /* printf("%s: rmdir %s\n", vp->cfp->logident,
-		 path); */
-	    }
 	    unweb(i, vp->orig[i]);
 	  }
 	}
