@@ -494,7 +494,7 @@ fprintf(stderr,"%s: Unknown option, c=%d ('%c')\n",progname,c,c);
 				progname, strerror(errno));
 			exit(1);
 		}
-		if (listen(s, 5) < 0) {
+		if (listen(s, 20) < 0) {
 			fprintf(stderr, "%s: listen(): %s\n",
 				progname, strerror(errno));
 			exit(1);
@@ -856,8 +856,10 @@ int insecure;
 		return;
 	}
 	for (cp = cp+5; *cp != '\0' && *cp != '<'; ++cp)
-		if (!isascii(*cp) || !isspace(*cp))
+		if (!isascii(*cp) || !isspace(*cp)) {
+			type(501, "where is <...> in that?");
 			return;
+		}
 	if (*cp == '\0') {
 		type(501, "where is <...> in that?");
 		return;
@@ -1041,8 +1043,10 @@ char *cp;
 		return;
 	}
 	for (cp = cp+3; *cp != '\0' && *cp != '<'; ++cp)
-		if (!isascii(*cp) || !isspace(*cp))
+		if (!isascii(*cp) || !isspace(*cp)) {
+			type(501, "where is <...> in that?");
 			return;
+		}
 	if (*cp == '\0') {
 		type(501, "where is <...> in that?");
 		return;
@@ -1330,13 +1334,14 @@ smtpserver(insecure)
 			type(550, "Syntax error");
 			continue;
 		}
-		if (*cp != '\0')
+		c = *cp;
+		if (c != '\0')
 			*cp = '\0';
 		for (carp = &command_list[0]; carp->verb != NULL; ++carp) {
 			if (cistrcmp(carp->verb, buf) == 0)
 				break;
 		}
-		*cp++ = ' ';
+		*cp++ = c;
 		if (carp->verb == NULL) {
 			type(550, "Unknown command '%s'", buf);
 #ifdef	LOG_WARNING
@@ -1488,8 +1493,6 @@ smtpserver(insecure)
 			debug_report(VerboseCommand,rhostname,buf);
 			break;
 		case Tick:
-			if (*cp != '\0')
-				*--cp = ' ';
 			type(250, "%s", buf);
 			with_protocol = WITH_BSMTP;
 			break;
@@ -1897,16 +1900,16 @@ router(function, holdlast, args)
 	char *buf, *holdcp;
 	int sawend;
 
+	if (args == NULL) {
+		type(501, (char *)NULL);
+		return NULL;
+	}
+
 	if (routerpid <= 0 && (routerpid = callr()) <= 0) {
 		type(451, (char *)NULL);
 		return NULL;
 	}
 
-	if (args == NULL
-	    || strchr(args, '\'') != NULL || args[strlen(args)-1] == '\\') {
-		type(501, (char *)NULL);
-		return NULL;
-	}
 	fprintf(tofp, "%s %s '%s'\n", ROUTER_SERVER, function, args);
 	fflush(tofp);
 	buf = holdcp = NULL;
@@ -2184,7 +2187,7 @@ mvdata(out,bad8bitp)
 #if EOF != -1
 		if (c == EOF) return EOF;
 #endif
-		if (--timercnt == 0) {
+		if (--timercnt <= 0) {
 		  /* Re-arm for every kilobyte */
 		  alarm(SMTP_DATA_TIME_PER_LINE);
 		  timercnt = 1024;
@@ -2258,7 +2261,7 @@ mvdata(out,bad8bitp)
 		if (c == EOF)		/* a little slower... */
 			return EOF;
 #endif
-		if (--timercnt == 0) {
+		if (--timercnt <= 0) {
 			/* Re-arm for every kilobyte */
 			alarm(SMTP_DATA_TIME_PER_LINE);
 			timercnt = 1024;
