@@ -121,14 +121,14 @@ struct rcpt *rp;
 
 	/* Allocate, and copy ! */
 
-	newcvt = (char **)malloc(sizeof(char *)*(hdrcnt+1));
+	newcvt = (char **)emalloc(sizeof(char *)*(hdrcnt+1));
 	if (newcvt != NULL) {
 
 	  char **ss = newcvt;
 	  probe = *(rp->newmsgheader);
 
 	  while (*probe) { /* Copy over */
-	    *ss = (char *)malloc(strlen(*probe)+1);;
+	    *ss = (char *)emalloc(strlen(*probe)+1);;
 	    if (!*ss) return 0; /* Out of memory ? */
 	    strcpy(*ss,*probe);
 	    ++hdrcnt;
@@ -183,7 +183,7 @@ append_header(va_alist)
 
 	while ((*hdrpp)[linecnt] != NULL) ++linecnt;
 
-	hdrp2 = (char**)realloc(*hdrpp,sizeof(char **) * (linecnt+3));
+	hdrp2 = (char**)erealloc(*hdrpp,sizeof(char **) * (linecnt+3));
 
 	if (!hdrp2) return -1;
 	hdrp2[linecnt] = (char*) emalloc(linelen+3);
