@@ -79,6 +79,12 @@
 #endif	/* !USE_SETEUID */
 #endif	/* !USE_SETREUID */
 
+#ifdef  USE_SETUIDX	/* IBM AIXism */
+# undef  setreuid
+# include <sys/id.h>
+# define setreuid(x,y)   setuidx(ID_REAL|ID_EFFECTIVE, y)
+#endif  /* !USE_SETUIDX */
+
 #ifdef  USE_TIMEVAL
 #include <sys/time.h>
 #else   /* !USE_TIMEVAL */
@@ -275,12 +281,12 @@ main(argc, argv)
 	int argc;
 	char *argv[];
 {
-	char file[MAXPATHLEN+1];
+	char file[MAXPATHLEN+128];
 	char *s;
 	int c, errflg, fd;
 	struct ctldesc *dp;
-	int matchhost = 0;	/* Match also the host-part when picking job */
 	char *host = NULL;	/* .. and what is my host ? */
+	int matchhost = 0;
 #if	defined(BIFF) || defined(RBIFF)
 	struct biffer *nbp;
 #endif
@@ -375,23 +381,41 @@ main(argc, argv)
 
 	getnobody();
 
-	while (!getout && fgets(file, sizeof file, stdin)) {
+	while (!getout) {
+
+	  /* Input:
+	       spool/file/name [ \t host.info ] \n
+	   */
+
+	  printf("#hungry\n");
+	  fflush(stdout);
+
+	  if (fgets(file, sizeof file, stdin) == NULL)
+	    break;
 	  if (emptyline(file, sizeof file))
 	    break;
+
+	  s = strchr(file,'\t');
+	  if (s != NULL) {
+	    if (host) free(host);
+	    host = strsave(s+1);
+	    *s = 0;
+	  }
+
 	  dp = ctlopen(file, channel, host, &getout, NULL, NULL);
-	  if (dp == NULL)
-	    continue;
-	  if (verboselog) {
-	    fclose(verboselog);
-	    verboselog = NULL;
+	  if (dp != NULL) {
+	    if (verboselog) {
+	      fclose(verboselog);
+	      verboselog = NULL;
+	    }
+	    if (dp->verbose) {
+	      verboselog = (FILE*)fopen(dp->verbose,"a");
+	      if (verboselog)
+		setbuf(verboselog,NULL);
+	    }
+	    process(dp);
+	    ctlclose(dp);
 	  }
-	  if (dp->verbose) {
-	    verboselog = (FILE*)fopen(dp->verbose,"a");
-	    if (verboselog)
-	      setbuf(verboselog,NULL);
-	  }
-	  process(dp);
-	  ctlclose(dp);
 	}
 
 #if	defined(BIFF) || defined(RBIFF)
@@ -1322,7 +1346,7 @@ creatembox(rp, filep, uid, gid, pw)
 	struct passwd *pw;
 {
 	char **maild;
-	int fd;
+	int fd = -1;
 
 	*uid = (short)pw->pw_uid;
 	*gid = (short)pw->pw_gid;
@@ -1933,7 +1957,6 @@ writemimeline(fp, buf, len, lastchp)
 	register char *cp, *s;
 	register int n;
 	int wlen, tlen, i = 0;
-	register char expect;
 	static char save = '\0';
 	static char frombuf[8];
 	static char *fromp = NULL;
