@@ -124,7 +124,7 @@ main(argc, argv)
 	char *argv[];
 {
 	char file[MAXPATHLEN+1];
-	char *channel, *host, *mailer, *cf;
+	char *channel, *host = NULL, *mailer, *cf;
 	struct ctldesc *dp;
 	int errflg, c;
 	struct maildesc *mp;
@@ -158,7 +158,7 @@ main(argc, argv)
 	    channel = optarg;
 	    break;
 	  case 'h':		/* remote hostname */
-	    host = optarg;
+	    host = strsave(optarg);
 	    break;
 	  case 'Q':
 	    mimeqpnarrow = 1;
@@ -193,13 +193,35 @@ main(argc, argv)
 	  exit(EX_OSFILE);
 	if (mp->flags & MO_REMOTEFROM)
 	  getmyuucpname(uucpname, sizeof uucpname);	/*XX*/
-	while (!getout && fgets(file, sizeof file, stdin)) {
+	while (!getout) {
+	  char *s;
+
+	  /* Input:
+	       spool/file/name [ \t host.info ] \n
+	   */
+
+	  printf("#hungry\n");
+	  fflush(stdout);
+
+	  if (fgets(file, sizeof file, stdin) == NULL)
+	    break;
 	  if (emptyline(file, sizeof file))
 	    break;
+
+	  s = strchr(file,'\t');
+	  if (s != NULL) {
+	    if (host) free(host);
+	    host = strsave(s+1);
+	    *s = 0;
+	  }
+
 	  ctlsticky((char *)NULL, (char *)NULL); /* reset */
 	  dp = ctlopen(file, channel, host, &getout, ctlsticky, NULL);
-	  if (dp == NULL)
+	  if (dp == NULL) {
+	    putc('\n',stdout);
+	    fflush(stdout);
 	    continue;
+	  }
 	  if (verboselog != stdout && verboselog != NULL) {
 	    fclose(verboselog);
 	    verboselog = NULL;
@@ -626,7 +648,7 @@ appendlet(dp, mp, fp, verboselog, convertmode)
 	  char iobuf[BUFSIZ];
 	  FILE *mfp = fdopen(mfd,"r");
 
-	  fseek(mfp,dp->msgbodyoffset);
+	  fseek(mfp,dp->msgbodyoffset,L_SET);
 
 	  setvbuf(mfp, iobuf, _IOFBF, sizeof(iobuf));
 
@@ -809,7 +831,6 @@ writemimeline(mp, fp, buf, len, convertmode, lastchp)
 	 */
 	register char *cp;
 	register int n;
-	register char expect;
 	static int  column;
 	register int qp_conv;
 	register int qp_chrs = 0;
@@ -862,7 +883,7 @@ writemimeline(mp, fp, buf, len, convertmode, lastchp)
 		if (putc('\r', fp) == EOF) return EOF;
 	      if (putc(c,fp) == EOF) return EOF;
 	      column = -1;
-	    } else if (expect != '\0') {
+	    } else {
 	      if (putc(c, fp) == EOF) return EOF;
 	    }
 	  } else if (convertmode == _CONVERT_8BIT) {
