@@ -26,6 +26,9 @@
 
 #include <stdio.h>
 #include "hostenv.h"
+#ifdef linux
+#define __USE_BSD 1
+#endif
 #include <ctype.h>
 #include <errno.h>
 #include <pwd.h>
@@ -57,11 +60,20 @@
 #endif	/* !USE_SETEUID */
 #endif	/* !USE_SETREUID */
 
+#ifdef  USE_SETUIDX	/* IBM AIXism */
+# undef  setreuid
+# include <sys/id.h>
+# define setreuid(x,y)   setuidx(ID_REAL|ID_EFFECTIVE, y)
+#endif  /* !USE_SETUIDX */
+
 #if	defined(TRY_AGAIN) && defined(USE_RESOLV)
 #define	BIND		/* Want BIND (named) nameserver support enabled */
 #endif	/* TRY_AGAIN */
 #ifdef	BIND
-#undef NOERROR				/* pinhead SunOS 5.1 */
+#ifdef NOERROR
+#undef NOERROR		/* Several SysV-streams using systems have NOERROR,
+			   which is not the same as  <arpa/nameser.h> has! */
+#endif
 #include <arpa/nameser.h>
 #include <resolv.h>
 
@@ -272,8 +284,8 @@ main(argc, argv)
 	int argc;
 	char *argv[];
 {
-	char file[MAXPATHLEN+1];
-	char *channel, *host;
+	char file[MAXPATHLEN+128];
+	char *channel = NULL, *host = NULL;
 	int i, fd, errflg, c, smtpstatus;
 	FILE *smtpfp[2];
 	struct ctldesc *dp;
@@ -323,8 +335,7 @@ main(argc, argv)
 	while ((c = getopt(argc, argv, "c:deh:l:rsxDET:VW78")) != EOF) {
 	  switch (c) {
 	  case 'c':		/* specify channel scanned for */
-	    channel = emalloc(strlen(optarg)+1);
-	    strcpy(channel, optarg);
+	    channel = strsave(optarg);
 	    break;
 	  case 'd':		/* turn on debugging output */
 	    ++debug;
@@ -343,8 +354,7 @@ main(argc, argv)
 	    strcpy(myhostname, optarg);
 	    break;
 	  case 'l':		/* log file */
-	    logfile = emalloc(strlen(optarg)+1);
-	    strcpy(logfile, optarg);
+	    logfile = strsave(optarg);
 	    break;
 	  case 'r':		/* use reserved port for SMTP connection */
 	    wantreserved = 1;
@@ -389,13 +399,14 @@ main(argc, argv)
 	    break;
 	  }
 	}
-	if (errflg || optind != argc - 1) {
+	if (errflg || optind >= argc) {
 	  fprintf(stderr,
-		  "Usage: %s [-c channel] host\n", argv[0]);
+		  "Usage: %s [-c channel] [host]\n", argv[0]);
 	  exit(EX_USAGE);
 	}
-	host = emalloc(strlen(argv[optind]) + 1);
-	strcpy(host, argv[optind]);
+	if (optind < argc) {
+	  host = strsave(argv[optind]);
+	}
 
 	if (myhostname[0] == 0) {
 	  /* Default it only when not having an explicite value
@@ -404,7 +415,7 @@ main(argc, argv)
 	  stashmyaddresses(myhostname);
 	}
 
-	if (conndebug && !debug) {
+	if (conndebug && !debug && host) {
 	  firstmx = 0;
 	  smtpconn(host, smtpfp, verboselog);
 	  exit(0);
@@ -428,12 +439,38 @@ main(argc, argv)
 	smtpstatus = EX_DATAERR;
 	smtp_isopen = 0;
 
-	while (!getout && fgets(file, sizeof file, stdin)) {
+	while (!getout) {
+	  /* Input:
+	       spool/file/name [ \t host.info ] \n
+	   */
+	  char *s;
+
+	  printf("#hungry\n");
+	  fflush(stdout);
+
+	  if (fgets(file, sizeof file, stdin) == NULL) break;
+
 	  if (emptyline(file, sizeof file))
 	    break;
 
+	  s = strchr(file,'\t');
+	  if (s != NULL) {
+	    *s++ = 0;
+	    if (host && strcmp(s,host) != 0) {
+	      if (smtp_isopen) {
+		fclose(smtpfp[0]); smtpfp[0] = NULL;
+		fclose(smtpfp[0]); smtpfp[0] = NULL;
+		if (logfp)
+		  fprintf(logfp, "%d#\t(closed SMTP channel)\n", pid);
+		smtp_isopen = 0;
+	      }
+	    }
+	    if (host) free(host);
+	    host = strsave(s);
+	  }
+
 	  if (debug > 1) { /* "DBGdiag:"-output */
-	    fprintf(stdout, "# (fdcnt=%d, file:%s)\n", countfds(), file);
+	    fprintf(stdout, "# (fdcnt=%d, file:%s, host:%s)\n", countfds(), file, host);
 	    fflush(stdout);
 	  }
 		       
@@ -446,6 +483,7 @@ main(argc, argv)
 	    dp = ctlopen(file, channel, host, &getout, NULL, matchroutermxes);
 	  if (dp == NULL)
 	    continue;
+
 	  if (verboselog) {
 	    fclose(verboselog);
 	    verboselog = NULL;
@@ -484,7 +522,7 @@ main(argc, argv)
 		fclose(verboselog);
 	if (logfp != NULL)
 		fclose(logfp);
-	if (smtpstatus = EX_DATAERR)
+	if (smtpstatus == EX_DATAERR)
 		smtpstatus = EX_OK; /* Nothing processed ?? */
 	exit(smtpstatus);
 	/* NOTREACHED */
@@ -562,7 +600,7 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 {
 	struct rcpt *rp = NULL;
 	int r, nrcpt, hsize, size, tout;
-	int content_kind, convertmode;
+	int content_kind = 0, convertmode;
 	int ascii_clean = 0;
 	struct stat stbuf;
 	char SMTPbuf[2000];
@@ -772,7 +810,7 @@ deliver(dp, smtpfp, verboselog, startrp, endrp)
 	      fprintf(logfp, "%d#\t(closed SMTP channel)\n", pid);
 	    smtp_isopen = 0;
 	  }
-	  return;
+	  return EX_TEMPFAIL;
 	}
 	/*
 	 * This is the one place where we *have* to wait forever because
@@ -1234,7 +1272,7 @@ char *buf;
 	  ehlo_capabilities |= ESMTP_SIZEOPT;
 	  ehlo_sizeval = -1;
 	  if (buf[5] == ' ')
-	    sscanf(buf+5,"%d",&ehlo_sizeval);
+	    sscanf(buf+5,"%ld",&ehlo_sizeval);
 	}
 	return 0;
 }
@@ -1565,7 +1603,7 @@ fdopen_fail:
 	      err = errno;
 	      if (fparr[0] != NULL) fclose(fparr[0]);
 	      if (fparr[1] != NULL) fclose(fparr[1]);
-printf("smtp: Failed to fdopen() a socket stream, errno=%d Hmm ??\n",err);
+printf("smtp: Failed to fdopen() a socket stream, errno=%d, err='%s' Hmm ??\n",err, strerror(err));
 fflush(stdout);
 abort();
 	    }
